@@ -45,7 +45,7 @@ struct config conf_template = {
 	max_changes:           DEF_CHANGES,
 	threshold_tune:        1,
 	always_diff:           0,
-	new_img:               1,
+	output_normal:         "on",
 	motion_img:            0,
 	output_all:            0,
 	gap:                   DEF_GAP,
@@ -55,20 +55,20 @@ struct config conf_template = {
 	input:                 IN_DEFAULT,
 	norm:                  0,
 	frame_limit:           DEF_MAXFRAMERATE,
-	mpeg:                  0,
 	quiet:                 0,
 	ppm:                   0,
 	noise:                 DEF_NOISELEVEL,
 	noise_tune:            1,
 	mingap:                0,
 	lightswitch:           0,
-	jpg_cleanup:           0,
 	nightcomp:             0,
-	adjustrate:            0,
-	oldlayout:             0,
 	low_cpu:               0,
 	nochild:               0,
 	autobright:            0,
+	brightness:	       0,
+	contrast:	       0,
+	saturation:	       0,
+	hue:		       0,
 	roundrobin_frames:     1,
 	roundrobin_skip:       1,
 	pre_capture:           0,
@@ -90,8 +90,6 @@ struct config conf_template = {
 	frequency:             0,
 	timelapse:             0,
 	timelapse_mode:        DEF_TIMELAPSE_MODE,
-	predict_threshold:     DEF_PREDICT_THRESHOLD,
-	predict_enable:        0,
 #ifdef __freebsd__
 	tuner_device:          TUNER_DEVICE,
 #endif
@@ -104,14 +102,13 @@ struct config conf_template = {
 	mpegpath:              DEF_MPEGPATH,
 	snappath:              DEF_SNAPPATH,
 	timepath:              DEF_TIMEPATH,
-	predictpath:           DEF_PREDICTPATH,
 	externcommand:         NULL,
 	mask_file:             NULL,
+	smart_mask_speed:      0,
 	sql_log_image:         1,
 	sql_log_snapshot:      1,
 	sql_log_mpeg:          0,
 	sql_log_timelapse:     0,
-	sql_log_prediction:    0,
 	mysql_db:              NULL,
 	mysql_host:            NULL,
 	mysql_user:            NULL,
@@ -127,7 +124,6 @@ struct config conf_template = {
 	pgsql_user:            NULL,
 	pgsql_password:        NULL,
 	pgsql_port:            5432,
-	mpeg_encode_bin:       "/usr/local/bin/mpeg_encode",
 	text_changes:          0,
 	text_left:             NULL,
 	text_right:            DEF_TIMESTAMP,
@@ -140,14 +136,12 @@ struct config conf_template = {
 
 static struct context **copy_bool(struct context **, char *, int);
 static struct context **copy_int(struct context **, char *, int);
-struct context **copy_string(struct context **, char *, int);
+//struct context **copy_string(struct context **, char *, int);
 static struct context **config_thread(struct context **cnt, char *str, int val);
-static struct context **load_desc(struct context **cnt, char *, int);
 
 static char *print_bool(struct context **, char **, int, int);
 static char *print_int(struct context **, char **, int, int);
 static char *print_string(struct context **, char **, int, int);
-static char *print_desc(struct context **, char **, int, int);
 static char *print_thread(struct context **, char **, int, int);
 
 static void usage(void);
@@ -243,23 +237,22 @@ config_param config_params[] = {
 	copy_int,
 	print_int
 	},
-#ifdef HAVE_CURL
 	{
 	"netcam_url",
 	"#URL to use if you are using a network camera, size will be autodetected (incl http://)\n"
-	"#Must be a URL that returns single jpeg pictures. Default: Not defined",
+	"#Must be a URL that returns single jpeg pictures or a raw mjpeg stream. Default: Not defined",
 	CONF_OFFSET(netcam_url),
 	copy_string,
 	print_string
 	},
 	{
 	"netcam_userpass",
-	"#Username and password for network camera (only if required). Default: not defined",
+	"#Username and password for network camera (only if required). Default: not defined\n"
+	"#Syntax is user:password",
 	CONF_OFFSET(netcam_userpass),
 	copy_string,
 	print_string
 	},
-#endif /* HAVE CURL */
 	{
 	"auto_brightness",
 	"#Let motion regulate the brightness of a video device (default: off)\n"
@@ -268,6 +261,38 @@ config_param config_params[] = {
 	copy_bool,
 	print_bool
 	},
+	{
+	"brightness",
+	"#Set the initial brightness of a video device.\n"
+	"#Valid range 0-255, default 0 = disabled",
+	CONF_OFFSET(brightness),
+	copy_int,
+	print_int
+	},
+	{
+	"contrast",
+	"#Set the contrast of a video device.\n"
+	"#Valid range 0-255, default 0 = disabled",
+	CONF_OFFSET(contrast),
+	copy_int,
+	print_int
+	},
+	{
+	"saturation",
+	"#Set the saturation of a video device.\n"
+	"#Valid range 0-255, default 0 = disabled",
+	CONF_OFFSET(saturation),
+	copy_int,
+	print_int
+	},
+	{
+	"hue",
+	"#Set the hue of a video device (NTSC feature).\n"
+	"#Valid range 0-255, default 0 = disabled",
+	CONF_OFFSET(hue),
+	copy_int,
+	print_int
+	},
 
 	{
 	"roundrobin_frames",
@@ -329,7 +354,7 @@ config_param config_params[] = {
 	{
 	"night_compensate",
 	"#Enables motion to adjust its detection/noise level for very dark frames\n"
-	"#don't mix with noise_tune (default: off)",
+	"#Don't use this with noise_tune on. (default: off)",
 	CONF_OFFSET(nightcomp),
 	copy_bool,
 	print_bool
@@ -353,6 +378,14 @@ config_param config_params[] = {
 	print_string
 	},
 	{
+	"smart_mask_speed",
+	"#Dynamically create a mask file during operation (default: 0)\n"
+	"#Adjust speed of mask changes from 0 (off) to 10 (fast)",
+	CONF_OFFSET(smart_mask_speed),
+	copy_int,
+	print_int
+	},
+	{
 	"lightswitch",
 	"#Ignore sudden massive light intensity changes given as a percentage of the picture\n"
 	"#area that changed intensity. Valid range: 0 - 100 , default: 0 = disabled",
@@ -402,7 +435,7 @@ config_param config_params[] = {
 	},
 	{
 	"max_mpeg_time",
-	"#Maximum length in seconds of an mpeg movie (common for ffmpeg and Berkeley types)\n"
+	"#Maximum length in seconds of an mpeg movie\n"
 	"#When value is exceeded a new mpeg file is created. (Default: 0 = infinite)",
 	CONF_OFFSET(maxmpegtime),
 	copy_int,
@@ -429,10 +462,13 @@ config_param config_params[] = {
 	"\n############################################################\n"
 	"# Image File Output\n"
 	"############################################################\n\n"
-	"#Output 'normal' pictures when motion is detected (default: on)",
-	CONF_OFFSET(new_img),
-	copy_bool,
-	print_bool
+	"#Output 'normal' pictures when motion is detected (default: on)\n"
+	"#Valid values: on, off, first\n"
+	"#When set to 'first', only the first picture of an event is saved.\n"
+	"#Can be used as preview shot for the corresponding movie.",
+	CONF_OFFSET(output_normal),
+	copy_string,
+	print_string
 	},
 	{
 	"output_motion",
@@ -513,7 +549,7 @@ config_param config_params[] = {
 	"ffmpeg_video_codec",
 	"#Codec to used by ffmpeg for the video compression.\n"
 	"#Timelapse mpegs are always made in mpeg1 format independent from this option.\n"
-	"#Supported formats are: mpeg1 (default), mpeg4, and msmpeg4.\n"
+	"#Supported formats are: mpeg1 (ffmpeg-0.4.8 only), mpeg4 (default), and msmpeg4.\n"
 	"#mpeg1 gives you files with extension .mpg\n"
 	"#mpeg4 or msmpeg4 give you files with extension .avi\n"
 	"#msmpeg4 is recommended for use with Windows Media Player because\n"
@@ -524,39 +560,6 @@ config_param config_params[] = {
 	},
 #endif /* HAVE_FFMPEG */
 
-	{
-	"mpeg_encode",
-	"\n############################################################\n"
-	"# Film (mpeg) File Output - Berkeley mpeg_encode Based\n"
-	"############################################################\n\n"
-	"#Use mpeg_encode from Berkeley to encode movies  (default: off)",
-	CONF_OFFSET(mpeg),
-	copy_bool,
-	print_bool
-	},
-	{
-	"mpeg_encode_bin",
-	"#Path to the mpeg_encode binary (default: /usr/bin/mpeg_encode)",
-	CONF_OFFSET(mpeg_encode_bin),
-	copy_string,
-	print_string
-	},
-	{
-	"adjust_rate",
-	"#Adjust the number of frames for an mpeg movie (only mpeg_encode) to get 25fps  (default: off)",
-	CONF_OFFSET(adjustrate),
-	copy_bool,
-	print_bool
-	},
-	{
-	"jpg_cleanup",
-	"#Delete jpeg images after they have been converted to movies by mpeg_encode (default: off)\n"
-	"#This option is only used when mpeg_encode is on",
-	CONF_OFFSET(jpg_cleanup),
-	copy_bool,
-	print_bool
-	},
-
 	{
 	"snapshot_interval",
 	"\n############################################################\n"
@@ -646,10 +649,7 @@ config_param config_params[] = {
 	"#Default: "DEF_JPEGPATH"\n"
 	"#Default value is equivalent to legacy oldlayout option\n"
 	"#For Motion 3.0 compatible mode choose: %Y/%m/%d/%H/%M/%S-%q\n"
-	"#File extension .jpg or .ppm is automatically added so do not include this\n"
-	"#For use of Berkeley mpeg_encode this parameter but be set as follows:\n"
-	"#If berkeley_single_directory = on: %v-%Y%m%d%H%M%S-%q\n"
-	"#If berkeley_single_directory = off: %Y/%m/%d/%H/%M/%S-%q",
+	"#File extension .jpg or .ppm is automatically added so do not include this\n",
 	CONF_OFFSET(jpegpath),
 	copy_string,
 	print_string
@@ -668,7 +668,7 @@ config_param config_params[] = {
 	},
 	{
 	"timelapse_filename",
-	"#File path for timelapse mpegs relative to target_dir (ffmpeg only)\n"
+	"#File path for timelapse mpegs relative to target_dir\n"
 	"#Default: "DEF_TIMEPATH"\n"
 	"#Default value is near equivalent to legacy oldlayout option\n"
 	"#For Motion 3.0 compatible mode choose: %Y/%m/%d-timelapse\n"
@@ -678,25 +678,6 @@ config_param config_params[] = {
 	print_string
 	},
 #endif /* HAVE_FFMPEG */
-	{
-	"berkeley_single_directory", 
-	"#Use single directory structure (For Berkeley mpeg_encode feature only)\n"
-	"#on = place mpeg_encode file in target_dir with name YYYYMMDDHHMMSS-n\n"
-	"#off (default) = place files as target_dir/YYYY/MM/DD/SS-n",
-	CONF_OFFSET(oldlayout),
-	copy_bool,
-	print_bool
-	},
-	{
-	"predict_filename",
-	"#File path for prediction files. (Default: "DEF_PREDICTPATH")\n"
-	"#Default value is equivalent to legacy oldlayout option\n"
-	"#For Motion 3.0 compatible mode choose: %Y/%m/%d/%H%M%S\n"
-	"#File extension .desc is automatically added so do not include this",
-	CONF_OFFSET(predictpath),
-	copy_string,
-	print_string
-	},
 
 	{
 	"webcam_port",
@@ -911,13 +892,7 @@ config_param config_params[] = {
 	copy_bool,
 	print_bool
 	},	
-	{
-	"sql_log_prediction",
-	"#Log to the database when creating a prediction file (default: off)",
-	CONF_OFFSET(sql_log_prediction),
-	copy_bool,
-	print_bool
-	},	
+
 #endif /* defined(HAVE_MYSQL) || defined(HAVE_PGSQL) */
 
 #ifdef HAVE_MYSQL
@@ -1014,32 +989,6 @@ config_param config_params[] = {
 	copy_string,
 	print_string
 	},
-
-	{
-	"predict_enable",
-	"\n############################################################\n"
-	"# Prediction Feature (experimental and incomplete)\n"
-	"############################################################\n\n"
-	"#Enable prediction",
-	CONF_OFFSET(predict_enable),
-	copy_bool,
-	print_bool
-	},
-	{
-	"predict_threshold",
-	"#Threshold for prediction (default: 90)",
-	CONF_OFFSET(predict_threshold),
-	copy_int,
-	print_int
-	},
-	{
-	"predict_description (default: not defined)",
-	"#Description for prediction",
-	0,
-	load_desc,
-	print_desc
-	},
-
 	{
 	"debug_parameter",
 	"\n############################################################\n"
@@ -1079,9 +1028,6 @@ static void conf_cmdline (struct context *cnt)
 			case 'a':
 				sscanf (optarg, "%d", &conf->snapshot_interval);
 				break;
-			case 'B':
-				conf->mpeg=1;
-				break;
 			case 'C':
 				conf->always_diff=1;
 				break;
@@ -1125,7 +1071,7 @@ static void conf_cmdline (struct context *cnt)
 				conf->motion_img=1;
 				break;
 			case 'N':
-				conf->new_img=0;
+				conf->output_normal = mystrcpy(conf->output_normal, optarg);
 				break;
 			case 'n':
 				if (!strcmp(optarg, "pal")) conf->norm=NORM_PAL;
@@ -1170,7 +1116,7 @@ static void conf_cmdline (struct context *cnt)
 				conf->motionvidpipe = mystrcpy(conf->motionvidpipe, optarg);
 				break;
 			case 'w':
-				conf->lightswitch=1;
+				sscanf (optarg, "%d", &conf->lightswitch);
 				break;
 			case 'h':
 			/* Do the same as an invallid option, but without the
@@ -1324,7 +1270,12 @@ void conf_print(struct context **cnt)
 			/*If config parameter has a value (not NULL) print it to the config file*/
 			if (val) {
 				fprintf(conffile, "%s\n", config_params[i].param_help);
-				fprintf(conffile, "%s %s\n\n", config_params[i].param_name, val);
+				/* If the option is a text_* and first char is a space put
+				   quotation marks around to allow leading spaces */
+				if (strncmp(config_params[i].param_name, "text", 4) || strncmp(val, " ", 1))
+					fprintf(conffile, "%s %s\n\n", config_params[i].param_name, val);
+				else
+					fprintf(conffile, "%s \"%s\"\n\n", config_params[i].param_name, val);
 			} else {
 				config_params[i].print(cnt, &val, i, thread);
 				/* It can either be a thread file parameter or a disabled parameter
@@ -1431,18 +1382,8 @@ struct context ** conf_load (struct context **cnt)
 	 */
 	i=-1;
 	while(cnt[++i])
-	{
 		conf_cmdline(cnt[i]);
 
-		/* If rotation is 90 or 270 degrees, then swap width and height
-		 * in the configuration struct. This way, all functions that rely
-		 * on the width and height will get the correct values. The 
-		 * functions that fetch video must swap them back, however.
-		 */
-		if((cnt[i]->conf.rotate_deg == 90) || (cnt[i]->conf.rotate_deg == 270))
-			myswap(cnt[i]->conf.width, cnt[i]->conf.height);
-	}
-
 	return cnt;
 }
 
@@ -1715,26 +1656,6 @@ static char *print_thread(struct context **cnt, char **str, int parm, int thread
 	return NULL;
 }
 
-static char *print_desc(struct context **cnt, char **str, int parm, int threadnr)
-{
-	char *retval=mymalloc(1);
-	struct predict_set *tmp;
-
-	if (!str || !retval)
-		return NULL;
-	if (!threadnr && cnt[1]!=NULL)
-		return NULL;
-
-	retval[0]=0;
-	for (tmp=cnt[threadnr]->predict_sets; tmp; tmp=tmp->next) {
-		retval=myrealloc(retval, strlen(retval)+strlen(tmp->filename)+22, "print_desc");
-		sprintf(retval+strlen(retval), "predict_description %s\n", tmp->filename);
-	}
-	*str=retval;
-
-	return NULL;
-}
-
 /* config_thread() is called during initial config file loading each time Motion
  * finds a thread option in motion.conf
  * The size of the context array is increased and the main context's values are
@@ -1802,67 +1723,6 @@ static struct context **config_thread(struct context **cnt, char *str, int val)
 	return cnt;
 }
 
-static struct context **load_desc(struct context **cnt, char *str, int val)
-{
-	struct predict_set *newset;
-	char line[256];
-	FILE *fp;
-	int i;
-
-	fp=fopen(str, "r");
-	if (!fp) {
-		syslog(LOG_ERR, "Couldn't open %s: %m", str);
-		return cnt;
-	}
-
-	newset=mymalloc(sizeof(struct predict_set));
-	memset(newset, 0x00, sizeof(struct predict_set));
-
-	if (fp) {
-		if (fgets(line, 256, fp)) {
-			stripnewline(line);
-			newset->name=mymalloc(strlen(line)+1);
-			strcpy(newset->name, line);
-			newset->filename=mymalloc(strlen(str)+1);
-			strcpy(newset->filename, str);
-		}
-		if (fgets(line, 256, fp)) {
-			sscanf(line, "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
-			    &newset->min.length,
-			    &newset->min.x,
-			    &newset->min.y,
-			    &newset->min.width,
-			    &newset->min.height,
-			    &newset->min.diffs,
-			    &newset->min.dx,
-			    &newset->min.dy
-			);
-		}
-		if (fgets(line, 256, fp)) {
-			sscanf(line, "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
-			    &newset->max.length,
-			    &newset->max.x,
-			    &newset->max.y,
-			    &newset->max.width,
-			    &newset->max.height,
-			    &newset->max.diffs,
-			    &newset->max.dx,
-			    &newset->max.dy
-			);
-		}
-		fclose(fp);
-		i=-1;
-		while (cnt[++i]) {
-			struct predict_set *tmpset=cnt[i]->predict_sets;
-			cnt[i]->predict_sets=mymalloc(sizeof(struct predict_set));
-			memcpy(cnt[i]->predict_sets, newset, sizeof(struct predict_set));
-			cnt[i]->predict_sets->next=tmpset;
-		}
-		free(newset);
-	}
-	return cnt;
-}
-
 static void usage (void)
 {
 	printf("motion Version "VERSION", Copyright 2000 Jeroen Vreeken\n");
@@ -1898,13 +1758,12 @@ static void usage (void)
 	printf("-V device\tvideo4linux video loopback input device for motion frames,\n");
 	printf("\t\tdefault: not set.");
 	printf("\nActions:\n");
-	printf("-B\t\tEncode all jpg images to mpeg after an event using mpeg_encode\n");
 	printf("-E command\tExecute 'command' when detecting motion.\n");
 	printf("-M address\tSend a mail to 'address' when detecting motion.\n");
 	printf("-O command\tExecute 'command' when saving an image.\n");
 	printf("-S nr\t\tSend a SMS to nr using sms_client when detecting motion.\n");
 	printf("-g seconds\tminimum gap between events, default %d\n", DEF_GAP);
 	printf("\n");
-	printf("Will read motion.conf from current directory, ~ and %s.\n", sysconfdir);
+	printf("Will read motion.conf from current directory, ~/.motion and %s.\n", sysconfdir);
 	printf("\n");
 }
