@@ -26,7 +26,7 @@ struct config {
 	int max_changes;
 	int threshold_tune;
 	int always_diff;
-	int new_img;
+	char *output_normal;
 	int motion_img;
 	int output_all;
 	int gap;
@@ -36,20 +36,20 @@ struct config {
 	int input;
 	int norm;
 	int frame_limit;
-	int mpeg;
 	int quiet;
 	int ppm;
 	int noise;
 	int noise_tune;
 	int mingap;
 	int lightswitch;
-	int jpg_cleanup;
 	int nightcomp;
-	int adjustrate;
-	int oldlayout; /* Only used for Berkeley mpeg_encode now */
 	int low_cpu;
 	int nochild;
 	int autobright;
+	int brightness;
+	int contrast;
+	int saturation;
+	int hue;
 	int roundrobin_frames;
 	int roundrobin_skip;
 	int pre_capture;
@@ -71,8 +71,6 @@ struct config {
 	int frequency;
 	int timelapse;
 	char *timelapse_mode; 
-	int predict_threshold;
-	int predict_enable;
 #ifdef __freebsd__
 	char *tuner_device;
 #endif
@@ -85,14 +83,13 @@ struct config {
 	char *mpegpath;
 	char *snappath;
 	char *timepath;
-	char *predictpath;
 	char *externcommand;
 	char *mask_file;
+	int smart_mask_speed;
 	int sql_log_image;
 	int sql_log_snapshot;
 	int sql_log_mpeg;
 	int sql_log_timelapse;
-	int sql_log_prediction;
 	char *mysql_db;
 	char *mysql_host;
 	char *mysql_user;
@@ -108,7 +105,6 @@ struct config {
 	char *pgsql_user;
 	char *pgsql_password;
 	int pgsql_port;
-	char *mpeg_encode_bin;
 	int text_changes;
 	char *text_left;
 	char *text_right;
@@ -148,5 +144,6 @@ void conf_print (struct context **);
 void malloc_strings (struct context *);
 char *mystrdup(char *);
 char *mystrcpy(char *, char *);
+struct context **copy_string(struct context **, char *, int);
 
 #endif /* _INCLUDE_CONF_H */
