@@ -20,6 +20,7 @@
 #include "event.h"
 #include "picture.h"
 #include "ffmpeg.h"
+#include "rotate.h"
 
 struct context **cnt=NULL;
 
@@ -36,46 +37,28 @@ typedef struct
 
 static void context_init (struct context *cnt)
 {
-	cnt->threadnr=0;
-
-	cnt->snapshot=0;
-	cnt->makemovie=0;
-	cnt->finish=0;
+	/*
+	* We first clear the entire structure to zero, then fill in any
+	* values which have non-zero default values.  Note that this
+	* assumes that a NULL address pointer has a value of binary 0
+	* (this is also assumed at other places within the code, i.e.
+	* there are instances of "if (ptr)").  Just for possible future
+	* changes to this assumption, any pointers which are intended
+	* to be initialised to NULL are listed within a comment.
+	*/
+  
+	memset(cnt, 0, sizeof(struct context));
 
 	cnt->noise=255;
-
-	cnt->shots=0;
-	cnt->event_nr=0;
-	cnt->prev_event=0;
 	cnt->currenttime=NULL;
-	cnt->lasttime=0;
-	cnt->eventtime=0;
-	cnt->lastshot=0;
 	cnt->lastrate=25;
-	cnt->filesc=0;
-	cnt->filesmc=0;
-	cnt->files=NULL;
-	cnt->filesm=NULL;
-	cnt->filesrate=NULL;
-	cnt->filesmrate=NULL;
 
 	memcpy(&cnt->track, &track_template, sizeof(struct trackoptions));
-	cnt->moved=0;
-
-	cnt->switched=0;
-	cnt->pause=0;
-	cnt->stream_count=0;
-
-	cnt->precap_nr=0;
-	cnt->precap_cur=0;
 
 	cnt->pipe=-1;
 	cnt->mpipe=-1;
 
-#if defined(HAVE_MYSQL) || defined(HAVE_PGSQL)
-	cnt->sql_mask=0;
-#endif
-	
+#if 0                 /* Here are the pointers required to be set */	
 #ifdef HAVE_MYSQL
 	cnt->database=NULL;
 #endif /* HAVE_MYSQL */
@@ -88,13 +71,8 @@ static void context_init (struct context *cnt)
 	cnt->ffmpeg_new=NULL;
 	cnt->ffmpeg_motion=NULL;
 	cnt->ffmpeg_timelapse=NULL;
-	cnt->newfilename[0] = '\0';
-	cnt->motionfilename[0] = '\0';
-	cnt->timelapsefilename[0] = '\0';
 #endif /* HAVE_FFMPEG */
-
-	cnt->predict_sets=NULL;
-	cnt->predict.length=0;
+#endif
 }
 
 
@@ -134,136 +112,11 @@ static char *get_pgm (FILE *picture, int width, int height)
 	return image;
 }
 
-static void cleanup (struct config *conf, char *filename)
-{
-	char tmp[PATH_MAX];
-
-	remove(filename);
-	if (!conf->oldlayout) {
-		strcpy(tmp, filename);
-		tmp[strlen(conf->filepath)+1+16]=0;
-		rmdir(tmp);
-		tmp[strlen(conf->filepath)+1+13]=0;
-		rmdir(tmp);
-		tmp[strlen(conf->filepath)+1+10]=0;
-		rmdir(tmp);
-		tmp[strlen(conf->filepath)+1+7]=0;
-		rmdir(tmp);
-		tmp[strlen(conf->filepath)+1+4]=0;
-		rmdir(tmp);
-	}
-}
-
-static void create_mpeg (struct context *cnt, int event_nr, int filesc, char **files, int *rate)
-{
-	int i, j;
-	char params[255];
-	char eventname[255];
-	char mpegfilename[255];
-	FILE *paramfile=NULL;
-	char *eventtime;
-	struct config *conf=&cnt->conf;
-
-	if (!filesc) return;
-	if (!fork()) {
-#ifndef WITHOUT_V4L
-		vid_close();
-#endif	
-		if (conf->oldlayout) {
-			eventtime=files[0]+strlen(cnt->conf.filepath)+1;
-			while (*eventtime++!='-');
-			strncpy(eventname, eventtime, 14);
-			if (files[0][strlen(files[0])-5]=='m') {
-				eventname[14]='m';
-				eventname[15]=0;
-			} else eventname[14]=0;
-		} else {
-			strncpy(eventname, files[0]+strlen(cnt->conf.filepath)+1, 11);/* daydir */
-			strncpy(eventname+11, files[0]+11+strlen(cnt->conf.filepath)+1, 2);/* hour */
-			strncpy(eventname+13, files[0]+14+strlen(cnt->conf.filepath)+1, 2);/* minute */
-			strncpy(eventname+15, files[0]+17+strlen(cnt->conf.filepath)+1, 2);/* second */
-			if (files[0][strlen(files[0])-5]=='m') {
-				eventname[17]='m';
-				eventname[18]=0;
-			} else eventname[17]=0;
-		}
-		sprintf(params, "%s/%s.params", cnt->conf.filepath, eventname);
-		sprintf(mpegfilename, "%s/%s.mpg", cnt->conf.filepath, eventname);
-		paramfile=myfopen(params, "w");
-		if(paramfile==NULL) {
-			syslog(LOG_ERR, "couldn't open %s\n", params);
-			exit(0);
-		}
-		fprintf(paramfile,
-			"PATTERN IBBPBBPBBPBBPBBP\n"
-			"OUTPUT %s\n"
-			"BASE_FILE_FORMAT ", mpegfilename);
-		if (!strncmp("ppm", files[0]+strlen(files[0])-3, 3))
-			fprintf(paramfile, "PPM\n");
-		else 
-			fprintf(paramfile, "JPEG\n");
-		fprintf(paramfile,
-			"INPUT_CONVERT *\n"
-			"GOP_SIZE 30\n"
-			"SLICES_PER_FRAME 1\n"
-			"INPUT_DIR /\n"
-			"INPUT\n");
-		for (i=0; i<filesc; i++) {
-			if (!conf->adjustrate || rate[i]>25) {
-				fprintf(paramfile, "%s\n", files[i]);
-			} else {
-				fprintf(stderr, "%d %d\n", rate[i], 25/(rate[i] ? rate[i]: 1));
-				for (j=0; j<25/(rate[i] ? rate[i]: 2); j++) {
-					fprintf(paramfile, "%s\n", files[i]);
-					fflush(paramfile);
-				}
-			}
-		}
-		fprintf(paramfile,
-			"END_INPUT\n"
-			"PIXEL HALF\n"
-			"RANGE 10\n"
-			"PSEARCH_ALG LOGARITHMIC\n"
-			"BSEARCH_ALG CROSS2\n"
-			"IQSCALE 4\n"
-			"PQSCALE 5\n"
-			"BQSCALE 13\n"
-			"FORCE_ENCODE_LAST_FRAME\n"
-			"REFERENCE_FRAME ORIGINAL\n"/*DECODED*/
-			"FRAME_RATE 25\n");
-		fclose(paramfile);
-
-		if (!fork()) {
-			nice(5);
-			execlp(cnt->conf.mpeg_encode_bin, "mpeg_encode", params, 0);
-			syslog(LOG_ERR, "Failed to start mpeg_encode: %m");
-			exit(1);
-		}
-		if (wait(0)==-1) {
-			syslog(LOG_ERR, "mpeg creation unsuccesfull: %m");
-			exit(1);
-		}
-		event(EVENT_FILECREATE, cnt, mpegfilename, (void *)FTYPE_MPEG, NULL);
-		for (i=0; i<filesc; i++) {
-			if (conf->jpg_cleanup) 
-				cleanup(conf, files[i]);
-			free(files[i]);
-		}
-		if (files)
-			free(files);
-		if (rate)
-			free(rate);
-		cleanup(conf, params);
-		exit(0);
-	}
-	return;
-}
-
 
 /*	Our SIGNAL-Handler
  *	We need this to handle alarms and external signals
  */
-static void sig_handler (int signo)
+static void sig_handler(int signo)
 {
 	int i;
 
@@ -344,8 +197,19 @@ static void sig_handler (int signo)
 	return;
 }
 
+/* The following function is a POSIX compliant replacement of the commonly used
+ * signal(SIGCHLD, SIG_IGN)
+ */
+static void sigchild_handler(int signo)
+{
+#ifdef WNOHANG
+	while (waitpid(-1, NULL, WNOHANG) > 0) {};
+#endif /* WNOHANG */
+	signal(SIGCHLD, sigchild_handler);
+	return;
+}
 
-static int motion_detected (struct context *cnt, int diffs, int dev, int devpipe, int devmpipe, char *newimg)
+static int motion_detected (struct context *cnt, int diffs, int dev, int devpipe, int devmpipe, unsigned char *newimg)
 {
 	struct config *conf=&cnt->conf;
 	struct images *imgs=&cnt->imgs;
@@ -362,7 +226,7 @@ static int motion_detected (struct context *cnt, int diffs, int dev, int devpipe
 				return 0;
 		}
 	}
-	alg_predict(&location, cnt, diffs, cnt->event_nr!=cnt->prev_event);
+
 	event(EVENT_MOTION, cnt, NULL, NULL, cnt->currenttime);
 	cnt->lasttime=cnt->currenttimep;
 	/* Take action if necessary */
@@ -406,7 +270,8 @@ static int motion_detected (struct context *cnt, int diffs, int dev, int devpipe
 			    cnt->imgs.new + (cnt->imgs.size * i),
 			    NULL, &tmptime);
 		}
-		
+		if (cnt->new_img==2)
+			cnt->preview_shot=1;
 		cnt->shots = tmpshots;
 	}
 
@@ -417,6 +282,7 @@ static int motion_detected (struct context *cnt, int diffs, int dev, int devpipe
 		/* Output the latest picture 'image_new' or image_out for motion picture.
 		 */
 		event(EVENT_IMAGE_DETECTED, cnt, newimg, NULL, cnt->currenttime);
+		cnt->preview_shot=0;
 	}
 
 	if (cnt->track.type != 0 && diffs != 0)
@@ -437,7 +303,10 @@ static void *motion_loop(void *arg)
 	FILE *picture;
 	int postcap=0;
 	int frame_buffer_size;
-
+	int smartmask_ratio=0;
+	int smartmask_count=20;
+	int smartmask_lastrate=0;
+	
 	int passflag=0;
 	long int *rolling_average_data;
 	long int rolling_average_limit, required_frame_time, frame_delay;
@@ -445,9 +314,18 @@ static void *motion_loop(void *arg)
 	struct timeval tv1, tv2;
 	unsigned long int rolling_average, elapsedtime, splittime, timenow=0, timebefore=0;
 
-	cnt->currenttime=&curtime;
+#ifdef HAVE_FFMPEG
+	/* Next two variables are used for timelapse feature
+	 * time_last_frame is set to 1 so that first comming timelapse or second=0
+	 * is acted upon.
+	 */
+	unsigned long int time_last_frame=1, time_current_frame;
+#endif /* HAVE_FFMPEG */
 
-	syslog(LOG_DEBUG, "Thread PID: %d\n", getpid());
+	cnt->currenttime=&curtime;
+	cnt->smartmask_speed=0;
+	
+	syslog(LOG_DEBUG, "Thread %d PID: %d\n", cnt->threadnr ,getpid());
 
 	if (!cnt->conf.filepath)
 		cnt->conf.filepath=".";
@@ -465,6 +343,7 @@ static void *motion_loop(void *arg)
 	if (!rolling_average_data)
 	{
 		syslog(LOG_ERR, "Unable to allocate memory [rolling_average_data]");
+		fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
 		exit(1);
 	}
 	
@@ -473,25 +352,45 @@ static void *motion_loop(void *arg)
 		rolling_average_data[j]=required_frame_time;
 
 	/* set the device settings */
-	cnt->video_dev=vid_start (cnt, cnt->conf.width, cnt->conf.height, cnt->conf.input, cnt->conf.norm, cnt->conf.frequency);
+	cnt->video_dev=vid_start (cnt);
 	if (cnt->video_dev==-1) {
 		syslog(LOG_ERR, "Capture error %s",strerror(errno));
+		syslog(LOG_INFO, "thread %d: finishing...", cnt->threadnr);
 		exit(1);
 	}
 	cnt->imgs.new=mymalloc(cnt->imgs.size);
+	memset(cnt->imgs.new, 0, cnt->imgs.size);       /* initialize to zero */
 	cnt->imgs.ref=mymalloc(cnt->imgs.size);
 	cnt->imgs.out=mymalloc(cnt->imgs.size);
+	cnt->imgs.smartmask=mymalloc(cnt->imgs.motionsize);
+	cnt->imgs.smartmask_final=mymalloc(cnt->imgs.motionsize);
+	cnt->imgs.smartmask_buffer=mymalloc(cnt->imgs.motionsize*sizeof(int));
 	cnt->imgs.labels=mymalloc(cnt->imgs.motionsize*sizeof(cnt->imgs.labels));
 	cnt->imgs.labelsize=mymalloc((cnt->imgs.motionsize/2+1)*sizeof(cnt->imgs.labelsize));
 	cnt->imgs.timestamp=mymalloc(sizeof(time_t));
 	cnt->imgs.shotstamp=mymalloc(sizeof(int));
 
+	/* Allocate a buffer for temp. usage in some places */
+	/* Only despeckle for now... */
+	cnt->imgs.common_buffer = malloc(3*cnt->imgs.width);
+
+	/* Now is a good time to init rotation data. Since vid_start has been
+ 	 * called, we know that we have imgs.width and imgs.height. When capturing
+	 * from a V4L device, these are copied from the corresponding conf values
+	 * in vid_start. When capturing from a netcam, they get set in netcam_start, 
+	 * which is called from vid_start.
+	 *
+	 * rotate_init will set cap_width and cap_height in cnt->rotate_data.
+	 */
+	rotate_init(cnt); /* rotate_deinit is called in main */
+
 	/* Allow videodevice to settle in */
 	sleep(1);
 
 	/* Capture first image, or we will get an alarm on start */
-	if (!(vid_next(cnt, cnt->video_dev, cnt->imgs.new, cnt->conf.width, cnt->conf.height))) {
-		syslog(LOG_ERR, "Capture error %s",strerror(errno));	
+	if (!(vid_next(cnt, cnt->imgs.new))) {
+		syslog(LOG_ERR, "Capture error %s",strerror(errno));
+        	fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
 		exit(1);
 	}
 
@@ -501,18 +400,22 @@ static void *motion_loop(void *arg)
 #ifndef WITHOUT_V4L
 	/* open video loopback devices if enabled */
 	if (cnt->conf.vidpipe) {
-		printf("Opening video loopback device for normal pictures\n");
-		cnt->pipe=vid_startpipe(cnt->conf.vidpipe, cnt->conf.width, cnt->conf.height, cnt->imgs.type);
+		fprintf(stderr,"Opening video loopback device for normal pictures\n");
+		/* vid_startpipe should get the output dimensions */
+		cnt->pipe=vid_startpipe(cnt->conf.vidpipe, cnt->imgs.width, cnt->imgs.height, cnt->imgs.type);
 		if (cnt->pipe < 0) {
 			syslog(LOG_ERR, "Failed to open video loopback\n");
+			fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
 			exit(1);
 		}
 	}
 	if (cnt->conf.motionvidpipe) {
-		printf("Opening video loopback device for motion pictures\n");
-		cnt->mpipe=vid_startpipe(cnt->conf.motionvidpipe, cnt->conf.width, cnt->conf.height, cnt->imgs.type);
+		fprintf(stderr,"Opening video loopback device for motion pictures\n");
+		/* vid_startpipe should get the output dimensions */
+		cnt->mpipe=vid_startpipe(cnt->conf.motionvidpipe, cnt->imgs.width, cnt->imgs.height, cnt->imgs.type);
 		if (cnt->mpipe < 0) {
 			syslog(LOG_ERR, "Failed to open video loopback\n");
+			fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
 			exit(1);
 		}
 	}
@@ -525,7 +428,8 @@ static void *motion_loop(void *arg)
 		if (!mysql_real_connect(cnt->database, cnt->conf.mysql_host, cnt->conf.mysql_user,
 		    cnt->conf.mysql_password, cnt->conf.mysql_db, 0, NULL, 0)) {
 			syslog(LOG_ERR, "Cannot connect to database %s on host %s with user %s", cnt->conf.mysql_db, cnt -> conf.mysql_host, cnt -> conf.mysql_user);
-			exit(1);
+			fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
+			exit(1);		
 		}
 	}
 #endif /* HAVE_MYSQL */
@@ -549,6 +453,7 @@ static void *motion_loop(void *arg)
 		if (PQstatus(cnt->database_pg) == CONNECTION_BAD)
 		{
 			syslog(LOG_ERR, "Connection to database '%s' failed: %s", cnt->conf.pgsql_db, PQerrorMessage(cnt->database_pg));
+			fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
 			exit(1);
 		}
 	}
@@ -557,22 +462,30 @@ static void *motion_loop(void *arg)
 	/* Load the mask file if any */
 	if (cnt->conf.mask_file) {
 		if ((picture=fopen(cnt->conf.mask_file, "r"))) {
-			cnt->imgs.mask=get_pgm(picture, cnt->conf.width, cnt->conf.height);
+			/* NOTE: The mask is expected to have the output dimensions. I.e., the mask
+			 * applies to the already rotated image, not the capture image. Thus, use
+			 * width and height from imgs.
+			 */
+			cnt->imgs.mask=get_pgm(picture, cnt->imgs.width, cnt->imgs.height);
 			fclose(picture);
 		} else {
 			syslog(LOG_ERR, "Error opening mask file %s: %m", cnt->conf.mask_file);
 		}
 		if (!cnt->imgs.mask) {
 			syslog(LOG_ERR, "failed to read mask image");
+			fprintf(stderr,"thread %d: finishing...\n", cnt->threadnr);
 			exit(1);
 		}
 	} else
 		cnt->imgs.mask=NULL;
 
-
-	/* Set noise level if no noice_tune */
-	if (!cnt->conf.noise_tune)
-		cnt->noise=cnt->conf.noise;
+	/* Always initialize smart_mask - someone could turn it on later... */
+	memset(cnt->imgs.smartmask, 0, cnt->imgs.motionsize);
+	memset(cnt->imgs.smartmask_final, 255, cnt->imgs.motionsize);
+	memset(cnt->imgs.smartmask_buffer, 0, cnt->imgs.motionsize*sizeof(int));
+	
+	/* Set noise level */
+	cnt->noise=cnt->conf.noise;
 
 	/* Set threshold value */
 	cnt->threshold=cnt->conf.max_changes;
@@ -583,15 +496,17 @@ static void *motion_loop(void *arg)
 	signal(SIGHUP, sig_handler);
 	signal(SIGTERM, sig_handler);
 	signal(SIGPIPE, SIG_IGN);
+	signal(SIGINT, sig_handler);
 
-	/* Initialize webcam server is webcam port is specified to not 0 */
-	if (cnt->conf.webcam_port)
+	/* Initialize webcam server if webcam port is specified to not 0 */
+	if (cnt->conf.webcam_port) {
 		if ( webcam_init(cnt) == -1 ) {	
-			syslog(LOG_ERR, "Problem enabling stream server: %s",strerror(errno));
-			printf("Problem enabling stream server: %s\n",strerror(errno));
-			cnt->finish=1; cnt->makemovie=0;
-                        //exit(0);
+			syslog(LOG_ERR, "Problem enabling stream server: %s", strerror(errno));
+			fprintf(stderr,"Problem enabling stream server: %s\n", strerror(errno));
+			cnt->finish=1;
+			cnt->makemovie=0;
 		}
+	}
 
 	/* Prevent first few frames from triggering motion... */
 	cnt->moved=8;
@@ -637,6 +552,30 @@ static void *motion_loop(void *arg)
 			}
 		}
 
+		/* Check for some parameter changes but only every second */
+		if (cnt->shots==0){
+			if (strcasecmp(cnt->conf.output_normal,"on") == 0)
+				cnt->new_img=1;
+			else if (strcasecmp(cnt->conf.output_normal,"first") == 0)
+				cnt->new_img=2;
+			else
+				cnt->new_img=0;
+			/* Sanity check for smart_mask_speed, silly value disables smart mask */
+			if (cnt->conf.smart_mask_speed < 0 || cnt->conf.smart_mask_speed > 10)
+				cnt->conf.smart_mask_speed = 0;
+			/* Has someone changed smart_mask_speed or framerate? */
+			if (cnt->conf.smart_mask_speed != cnt->smartmask_speed || smartmask_lastrate != cnt->lastrate){
+				if (cnt->conf.smart_mask_speed==0){
+					memset(cnt->imgs.smartmask, 0, cnt->imgs.motionsize);
+					memset(cnt->imgs.smartmask_final, 255, cnt->imgs.motionsize);
+				}
+				smartmask_lastrate=cnt->lastrate;
+				cnt->smartmask_speed=cnt->conf.smart_mask_speed;
+				/* Decay delay - based on smart_mask_speed (framerate independent)
+				   This is always 5*smartmask_speed seconds */
+				smartmask_ratio=5*cnt->lastrate*(11-cnt->smartmask_speed);
+			}
+		}
 		/* Get time for current frame */
 		cnt->currenttimep=time(NULL);
 		
@@ -650,7 +589,7 @@ static void *motion_loop(void *arg)
 		 * is used as the ffmpeg framerate when motion is detected.
 		 */
 		if (lastframe!=cnt->currenttimep) {
-			cnt->lastrate=cnt->shots;
+			cnt->lastrate=cnt->shots + 1;
 			cnt->shots=-1;
 			lastframe=cnt->currenttimep;
 		}
@@ -674,7 +613,7 @@ static void *motion_loop(void *arg)
 			cnt->precap_cur=0;
 
 		/* Fetch next frame from camera */
-		if (!vid_next(cnt, cnt->video_dev, newimg, cnt->conf.width, cnt->conf.height))
+		if (!vid_next(cnt, newimg))
 			break;
 		
 		/* Get current time and preserver last time for frame interval calc. */
@@ -691,8 +630,7 @@ static void *motion_loop(void *arg)
 		cnt->sql_mask = cnt->conf.sql_log_image * (FTYPE_IMAGE + FTYPE_IMAGE_MOTION) +
 		                cnt->conf.sql_log_snapshot * FTYPE_IMAGE_SNAPSHOT +
 		                cnt->conf.sql_log_mpeg * (FTYPE_MPEG + FTYPE_MPEG_MOTION) +
-		                cnt->conf.sql_log_timelapse * FTYPE_MPEG_TIMELAPSE +
-		                cnt->conf.sql_log_prediction * FTYPE_PREDICT;
+		                cnt->conf.sql_log_timelapse * FTYPE_MPEG_TIMELAPSE;
 		
 #endif /* defined(HAVE_MYSQL) || defined(HAVE_PGSQL) */
 
@@ -703,30 +641,34 @@ static void *motion_loop(void *arg)
 		if (cnt->threshold && !cnt->pause) {
 			/* if we've already detected motion and we want to see if there's
 			 * still motion, don't bother trying the fast one first. IF there's
-			 * motion, the alg_diff_hybrid will trigger alg_diff_standard
+			 * motion, the alg_diff will trigger alg_diff_standard
 			 * anyway
 			 */
 			if (detecting_motion)
 				diffs=alg_diff_standard(cnt, newimg);
 			else
 				diffs=alg_diff(cnt, newimg);
- 
- 			/* Despeckle feature is run now */
+
+			/* Despeckle feature is run now */
 			if (cnt->conf.despeckle && diffs > 0) {
 				int olddiffs = diffs;
 				diffs=alg_despeckle(cnt);
 				if (cnt->conf.always_diff)
-					printf("Despeckle %d (%s) from %d to %d\n",
+					fprintf(stderr,"Despeckle %d (%s) from %d to %d\n",
 						cnt->threadnr, cnt->conf.despeckle,
 						olddiffs, diffs);
 			}
 		} else
 			diffs=0;
-#ifndef WITHOUT_V4L			
-		/* Auto-brightness feature */
-		if (cnt->conf.autobright)
-			vid_autobright(&cnt->conf, cnt->video_dev, newimg, cnt->conf.width, cnt->conf.height);
-#endif
+
+		/* Manipulate smart_mask sensitivity (only every smartmask_ratio seconds) */
+		if (cnt->smartmask_speed){
+			if (!--smartmask_count){
+				tune_smartmask(cnt);
+				smartmask_count=smartmask_ratio;
+			}
+		}
+		
 		if (cnt->moved) {
 			cnt->moved--;
 			diffs=0;
@@ -738,7 +680,7 @@ static void *motion_loop(void *arg)
 		 */
 		if (cnt->conf.lightswitch) {
 			if (alg_lightswitch(cnt, diffs)) {
-				printf("Lightswitch detected\n");
+				fprintf(stderr,"Lightswitch detected\n");
 				if (cnt->moved<5)
 					cnt->moved = 5;
 				diffs = 0;
@@ -761,44 +703,13 @@ static void *motion_loop(void *arg)
 				
 				/* if tracking is enabled we track a little */
 				if (cnt->track.type) {
-					printf("%d %d %d\n", cnt->prev_event, cnt->event_nr, cnt->makemovie);
+					fprintf(stderr,"%d %d %d\n", cnt->prev_event, cnt->event_nr, cnt->makemovie);
 					cnt->moved=track_center(cnt, cnt->video_dev, 0, 0, 0);
 				}
 				
 				cnt->makemovie=0;
 				cnt->event_nr++;
 				
-				/* make mpeg using old Berkeley mpeg_encode */
-				if (cnt->conf.mpeg) {
-					if (cnt->filesc)
-						create_mpeg(cnt, cnt->prev_event,
-							    cnt->filesc,
-							    cnt->files, 
-							    cnt->filesrate
-							    );
-					if (cnt->filesmc)
-						create_mpeg(cnt, cnt->prev_event,
-							    cnt->filesmc, 
-							    cnt->filesm, 
-							    cnt->filesmrate
-							    );
-				}
-				for (i=0; i<cnt->filesc; i++) free(cnt->files[i]);
-				if (cnt->files)
-					free(cnt->files);
-				for (i=0; i<cnt->filesmc; i++) free(cnt->filesm[i]);
-				if (cnt->filesm)
-					free(cnt->filesm);
-				if (cnt->filesrate)
-					free(cnt->filesrate);
-				if (cnt->filesmrate)
-					free(cnt->filesmrate);
-				cnt->filesc=0;
-				cnt->files=NULL;
-				cnt->filesmc=0;
-				cnt->filesm=NULL;
-				cnt->filesrate=NULL;
-				cnt->filesmrate=NULL;
 			}
 		}
 
@@ -811,23 +722,23 @@ static void *motion_loop(void *arg)
 
 		/* Add changed pixels in upper right corner of the pictures */
 		if (cnt->conf.text_changes){
-			char tmp[10];
+			char tmp[15];
 			sprintf(tmp, "%d", diffs);
-			draw_text(newimg, cnt->conf.width-10, 10, cnt->conf.height, cnt->conf.width, tmp);
+			draw_text(newimg, cnt->imgs.width-10, 10, cnt->imgs.height, cnt->imgs.width, tmp);
 		}
 
 		/* Add text in lower left corner of the pictures */
 		if (cnt->conf.text_left) {
 			char tmp[PATH_MAX];
 			mystrftime(tmp, sizeof(tmp), cnt->conf.text_left, cnt->currenttime, cnt->event_nr, cnt->shots);
-			draw_text(newimg, 10, cnt->conf.height-10, cnt->conf.height, cnt->conf.width, tmp);
+			draw_text(newimg, 10, cnt->imgs.height-10, cnt->imgs.height, cnt->imgs.width, tmp);
 		}
 
 		/* Add text in lower right corner of the pictures */
 		if (cnt->conf.text_right) {
 			char tmp[PATH_MAX];
 			mystrftime(tmp, sizeof(tmp), cnt->conf.text_right, cnt->currenttime, cnt->event_nr, cnt->shots);
-			draw_text(newimg, cnt->conf.width-10, cnt->conf.height-10, cnt->conf.height, cnt->conf.width, tmp);
+			draw_text(newimg, cnt->imgs.width-10, cnt->imgs.height-10, cnt->imgs.height, cnt->imgs.width, tmp);
 		}
 
 		/* Is output_all enabled?
@@ -867,10 +778,10 @@ static void *motion_loop(void *arg)
 				alg_noise_tune(cnt, newimg);
 			}
 			/* if motion was detected, reset the noise-level */
-			else
-			{
-				cnt->noise = (cnt->noise + cnt->conf.noise) / 2;
-			}
+			//else
+			//{
+			//	cnt->noise = (cnt->noise + cnt->conf.noise) / 2;
+			//}
 		}
 
 		/* if we are not noise tuning lets make sure that remote controlled
@@ -890,7 +801,7 @@ static void *motion_loop(void *arg)
 
 		/* If enabled output number of motion detected pixels to console */
 		if (cnt->conf.always_diff)
-			printf("changes: %d\n", diffs);
+			fprintf(stderr,"changes: %d\n", diffs);
 
 		/* Prevent the motion created by moving camera or sudden light intensity
 		 * being detected by creating a fresh reference frame
@@ -910,69 +821,78 @@ static void *motion_loop(void *arg)
 			event(EVENT_IMAGE_SNAPSHOT, cnt, newimg, NULL, cnt->currenttime);
 			cnt->snapshot=0;
 		}
-		
-#ifdef HAVE_FFMPEG
-		/* Timelapse feature */ 
-		if (cnt->conf.timelapse) {			
 
-			/* If ffmpeg timelapse is enabled and time since epoch MOD ffmpeg_timelaps = 0
-			 * add a timelapse frame to the timelapse mpeg.
-			 */
-			if (cnt->shots == 0 && mktime(cnt->currenttime)%cnt->conf.timelapse==0)
-				event(EVENT_TIMELAPSE, cnt, newimg, NULL, cnt->currenttime);
-	
+#ifdef HAVE_FFMPEG
+		
+		time_current_frame = mktime(cnt->currenttime);
+ 		
+		if (cnt->conf.timelapse) {
+
 			/* Check to see if we should start a new timelapse file. We start one when 
-			   we are on the first shot, and and the seconds are zero. We must use the seconds
-			   to prevent the timelapse file from getting reset multiple times during the minute.
-			*/ 
-			if (cnt->currenttime->tm_min == 0 && cnt->currenttime->tm_sec == 0 && cnt->shots == 0) {
+			 * we are on the first shot, and and the seconds are zero. We must use the seconds
+			 * to prevent the timelapse file from getting reset multiple times during the minute.
+			 */ 
+			if (cnt->currenttime->tm_min == 0 && (time_current_frame % 60 < time_last_frame % 60) && cnt->shots == 0) {
 				
 				if (strcasecmp(cnt->conf.timelapse_mode,"manual") == 0)
 				;/* No action */ 
-
+				
 				/* If we are daily, raise timelapseend event at midnight */ 
 				else if (strcasecmp(cnt->conf.timelapse_mode,"daily") == 0) {
 					if (cnt->currenttime->tm_hour == 0)
 						event(EVENT_TIMELAPSEEND, cnt, NULL, NULL, cnt->currenttime);
 				}
-
+				
 				/* handle the hourly case */ 	
 				else if (strcasecmp(cnt->conf.timelapse_mode,"hourly") == 0) {
 					event(EVENT_TIMELAPSEEND, cnt, NULL, NULL, cnt->currenttime);
 				}
-
+				
 				/* If we are weekly-sunday, raise timelapseend event at midnight on sunday */ 
 				else if (strcasecmp(cnt->conf.timelapse_mode,"weekly-sunday") == 0) {
-					if (cnt->currenttime->tm_hour == 0 && cnt->currenttime->tm_wday == 0)
+					if (cnt->currenttime->tm_wday == 0 && cnt->currenttime->tm_hour == 0)
 						event(EVENT_TIMELAPSEEND, cnt, NULL, NULL, cnt->currenttime);
 				}
 				
 				/* If we are weekly-monday, raise timelapseend event at midnight on monday */ 
 				else if (strcasecmp(cnt->conf.timelapse_mode,"weekly-monday") == 0) {
-					if (cnt->currenttime->tm_hour == 0 && cnt->currenttime->tm_wday == 1)
+					if (cnt->currenttime->tm_wday == 1 && cnt->currenttime->tm_hour == 0)
 						event(EVENT_TIMELAPSEEND, cnt, NULL, NULL, cnt->currenttime);
 				}
 				
-				/* If we are weekly-monday, raise timelapseend event at midnight on monday */
+				/* If we are monthly, raise timelapseend event at midnight on first day of month */
 				else if (strcasecmp(cnt->conf.timelapse_mode,"monthly") == 0) {
-					if (cnt->currenttime->tm_mday == 0 && cnt->currenttime->tm_hour == 0)
+					if (cnt->currenttime->tm_mday == 1 && cnt->currenttime->tm_hour == 0)
 						event(EVENT_TIMELAPSEEND, cnt, NULL, NULL, cnt->currenttime);
 				}
 				
 				/* If invalid we report in syslog once and continue in manual mode */
 				else {
 					syslog(LOG_ERR, "invalid timelapse_mode argument '%s'",cnt->conf.timelapse_mode);
+					fprintf(stderr,"invalid timelapse_mode argument '%s'\n",cnt->conf.timelapse_mode);
 					syslog(LOG_ERR, "defaulting to manual timelapse mode");
+					fprintf(stderr,"defaulting to manual timelapse mode,\n");
 					conf_cmdparse(&cnt, "ffmpeg_timelapse_mode","manual");
 				}
 			}
+			
+			/* If ffmpeg timelapse is enabled and time since epoch MOD ffmpeg_timelaps = 0
+			 * add a timelapse frame to the timelapse mpeg.
+			 */
+			if (cnt->shots == 0 &&
+				time_current_frame % cnt->conf.timelapse <= time_last_frame % cnt->conf.timelapse)	
+				event(EVENT_TIMELAPSE, cnt, newimg, NULL, cnt->currenttime);
 		}
+		
 		/* if timelapse mpeg is in progress but conf.timelapse is zero then close timelapse file
 		 * This is an important feature that allows manual roll-over of timelapse file using xmlrpc
 		 * via a cron job
 		 */
 		else if (cnt->ffmpeg_timelapse)
 			event(EVENT_TIMELAPSEEND, cnt, NULL, NULL, cnt->currenttime);
+			
+		time_last_frame = time_current_frame;
+		
 #endif /* HAVE_FFMPEG */
 
 		/* feed last image and motion image to video device pipes */
@@ -1005,7 +925,7 @@ static void *motion_loop(void *arg)
 				frame_delay=required_frame_time;
 			/* Commented out - only for special debugging
 			if (!cnt->conf.quiet)
-				printf("Split Time is %7lu. Applying %6lu delay. Avg Frame Time %7lu Frame Error %ld\n",
+				fprintf(stderr,"Split Time is %7lu. Applying %6lu delay. Avg Frame Time %7lu Frame Error %ld\n",
 				    elapsedtime, frame_delay, rolling_average, rolling_average-required_frame_time);
 			*/
 			usleep(frame_delay);
@@ -1026,8 +946,11 @@ static void *motion_loop(void *arg)
 			timenow=(tv1.tv_usec+(1000000L/cnt->conf.low_cpu)*tv1.tv_sec)-required_frame_time;
 		}
 	}
+
+	if (rolling_average_data) 
+		free(rolling_average_data); 
 	
-	printf("Thread exiting\n");
+	fprintf(stderr,"Thread %d exiting\n",cnt->threadnr);
 	if (!cnt->finish)
 		syslog(LOG_ERR, "Somebody stole the video device, lets hope we got his picture");
 	event(EVENT_STOP, cnt, NULL, NULL, NULL);
@@ -1059,7 +982,7 @@ int main (int argc, char **argv)
 	cnt[1]=NULL;
 
 	/* Enable automatic zombie reaping */
-	signal(SIGCHLD, SIG_IGN);
+	signal(SIGCHLD, sigchild_handler);
 
 	/* Command line arguments are being pointed to from cnt[0] and we call conf_load which loads
 	 * the config options from motion.conf, thread config files and the command line.
@@ -1074,7 +997,7 @@ int main (int argc, char **argv)
 
 	if (cnt[0]->daemon) {
 		if (fork()) {
-			printf("motion: going to daemon mode\n");
+			fprintf(stderr,"motion: going to daemon mode\n");
 			exit(0);
 		}
 		signal(SIGQUIT, sig_handler);
@@ -1126,7 +1049,7 @@ int main (int argc, char **argv)
 			context_init(cnt[0]);
 			cnt[1]=NULL;
 
-			signal(SIGCHLD, SIG_IGN);
+			signal(SIGCHLD, sigchild_handler);
 
 			cnt[0]->conf.argv=argv;
 			cnt[0]->conf.argc=argc;
@@ -1142,14 +1065,14 @@ int main (int argc, char **argv)
 		i = 0;
 		while (cnt[++i]) {
 			webcam_port = cnt[i]->conf.webcam_port;
-			printf("Webcam port for thread %d is %d\n", i, webcam_port);
+			fprintf(stderr,"Webcam port for thread %d is %d\n", i, webcam_port);
 			j = i;
 			while (cnt[++j]) {
 				if (cnt[j]->conf.webcam_port == webcam_port && webcam_port != 0) {
 					cnt[j]->conf.webcam_port = 0;
-					printf("Webcam port number %d for thread %d conflicts with thread %d\n",
+					fprintf(stderr,"Webcam port number %d for thread %d conflicts with thread %d\n",
 					        webcam_port, j, i);
-					printf("Webcam feature for thread %d is disabled.\n", j);
+					fprintf(stderr,"Webcam feature for thread %d is disabled.\n", j);
 					syslog(LOG_ERR, "Webcam port number %d for thread %d conflicts with thread %d\n",
 					       webcam_port, j, i);
 					syslog(LOG_ERR, "Webcam feature for thread %d is disabled.\n", j);	
@@ -1162,7 +1085,7 @@ int main (int argc, char **argv)
 			/* first cnt is global if 'thread' option is used */
 			if (!i && cnt[i+1])
 				continue;
-			printf("Thread%d device: %s input: %d\n",
+			fprintf(stderr,"Thread%d device: %s input: %d\n",
 			    cnt[i]->threadnr,
 			    cnt[i]->conf.netcam_url?cnt[i]->conf.netcam_url:cnt[i]->conf.video_device,
 			    cnt[i]->conf.netcam_url?-1:cnt[i]->conf.input
@@ -1199,8 +1122,33 @@ int main (int argc, char **argv)
 			free(cnt[i]->imgs.ref);
 		if (cnt[i]->imgs.new)
 			free(cnt[i]->imgs.new);
+		if (cnt[i]->imgs.labels)
+			free(cnt[i]->imgs.labels);
+		if (cnt[i]->imgs.labelsize)
+			free(cnt[i]->imgs.labelsize);
+		if (cnt[i]->imgs.smartmask)
+			free(cnt[i]->imgs.smartmask);
+		if (cnt[i]->imgs.smartmask_final)
+			free(cnt[i]->imgs.smartmask_final);
+		if (cnt[i]->imgs.smartmask_buffer)
+			free(cnt[i]->imgs.smartmask_buffer);
+		if (cnt[i]->imgs.common_buffer)
+			free(cnt[i]->imgs.common_buffer);
+		if (cnt[i]->imgs.timestamp)
+			free(cnt[i]->imgs.timestamp);
 		if (cnt[i]->imgs.shotstamp)
-			free(cnt[i]->imgs.shotstamp); 
+			free(cnt[i]->imgs.shotstamp);
+		for (j=0; config_params[j].param_name != NULL; j++) {
+			if (config_params[j].copy == copy_string) {
+				void **val;
+				val=(void *)cnt[i]+config_params[j].conf_value;
+				if (*val) {
+					free(*val);
+					*val = NULL;
+				}
+			}
+		} 
+		rotate_deinit(cnt[i]); /* cleanup image rotation data */
 		if (cnt[i])
 			free(cnt[i]);
 	}
@@ -1221,7 +1169,7 @@ void * mymalloc(size_t nbytes)
 {
 	void *dummy = malloc(nbytes);
 	if (!dummy) {
-		syslog(LOG_EMERG, "Could not allocate %d bytes of memory!", nbytes);
+		syslog(LOG_EMERG, "Could not allocate %zd bytes of memory!", nbytes);
 		exit(1);
 	}
 
@@ -1238,13 +1186,13 @@ void * myrealloc(void *ptr, size_t size, char *desc)
 	if (size == 0)
 	{
 		free(ptr);
-		syslog(LOG_WARNING, "Warning! Function %s tries to resize memoryblock at %p to %d bytes!", desc, ptr, size);
+		syslog(LOG_WARNING, "Warning! Function %s tries to resize memoryblock at %p to %zd bytes!", desc, ptr, size);
 	}
 	else
 	{
 		dummy = realloc(ptr, size);
 		if (!dummy) {
-			syslog(LOG_EMERG, "Could not resize memory-block at offset %p to %d bytes (function %s)!", ptr, size, desc);
+			syslog(LOG_EMERG, "Could not resize memory-block at offset %p to %zd bytes (function %s)!", ptr, size, desc);
 			exit(1);
 		}
 	}
