@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#define __USE_GNU
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -84,9 +85,9 @@
 
 #define DEF_FFMPEG_BPS      400000
 #define DEF_FFMPEG_VBR           0
-#define DEF_FFMPEG_CODEC   "mpeg1"
+#define DEF_FFMPEG_CODEC   "mpeg4"
 
-#define DEF_PREDICT_THRESHOLD   90
+//#define DEF_PREDICT_THRESHOLD   90
 #define THRESHOLD_TUNE_LENGTH  256
 
 #define DEF_MAXSTREAMS          10  /* Maximum number of webcam clients per camera */
@@ -98,7 +99,6 @@
 #define DEF_JPEGPATH       "%v-%Y%m%d%H%M%S-%q"
 #define DEF_MPEGPATH       "%v-%Y%m%d%H%M%S"
 #define DEF_TIMEPATH       "%Y%m%d-timelapse"
-#define DEF_PREDICTPATH    "%v-%Y%m%d%H%M%S"
 
 #define DEF_TIMELAPSE_MODE "daily"
 
@@ -109,13 +109,34 @@
 #define FTYPE_MPEG             8
 #define FTYPE_MPEG_MOTION     16
 #define FTYPE_MPEG_TIMELAPSE  32
-#define FTYPE_PREDICT         64
 
 #define FTYPE_MPEG_ANY    (FTYPE_MPEG | FTYPE_MPEG_MOTION | FTYPE_MPEG_TIMELAPSE)
 #define FTYPE_IMAGE_ANY   (FTYPE_IMAGE | FTYPE_IMAGE_SNAPSHOT | FTYPE_IMAGE_MOTION)
 
-/* swap two variables using the xor algorithm */
-#define myswap(x, y) do { x = ((x) ^ (y)); y = ((x) ^ (y)); x = ((x) ^ (y)); } while(0) 
+/* DIFFERENCES BETWEEN imgs.width, conf.width AND rotate_data.cap_width
+ * (and the corresponding height values, of course)
+ * ===========================================================================
+ * Location      Purpose
+ * 
+ * conf          The values in conf reflect width and height set in the
+ *               configuration file. These can be set via xmlrpc, but they are
+ *               not used internally by Motion, so it won't break anything.
+ *               These values are transferred to imgs in vid_start.
+ *
+ * imgs          The values in imgs are the actual output dimensions. Normally
+ *               the output dimensions are the same as the capture dimensions,
+ *               but for 90 or 270 degrees rotation, they are not. E.g., if 
+ *               you capture at 320x240, and rotate 90 degrees, the output
+ *               dimensions are 240x320.
+ *               These values are set from the conf values in vid_start, or 
+ *               from the first JPEG image in netcam_start. For 90 or 270 
+ *               degrees rotation, they are swapped in rotate_init.
+ *
+ * rotate_data   The values in rotate_data are named cap_width and cap_height,
+ *               and contain the capture dimensions. The difference between
+ *               capture and output dimensions is explained above.
+ *               These values are set in rotate_init.
+ */
 
 /* date/time drawing, draw.c */
 int draw_text (char *image, int startx, int starty, int height, int width, char *text);
@@ -125,6 +146,10 @@ struct images {
 	unsigned char *ref;
 	unsigned char *out;
 	unsigned char *mask;
+	unsigned char *smartmask;
+	unsigned char *smartmask_final;
+	unsigned char *common_buffer;
+	int *smartmask_buffer;
 	int *labels;
 	int *labelsize;
 	int width;
@@ -139,8 +164,57 @@ struct images {
 	int *shotstamp;
 };
 
+/* Contains data for image rotation, see rotate.c. */
+struct rotdata {
+	/* Temporary buffer for 90 and 270 degrees rotation. */
+	char *temp_buf;
+	/* Pointer to a rotate-180-degrees function. */
+	void (*rotate_180)(char *, int);
+	/* Degrees to rotate; copied from conf.rotate_deg. This is the value
+	 * that is actually used. The value of conf.rotate_deg cannot be used
+	 * because it can be changed by motion-control, and changing rotation
+	 * while Motion is running just causes problems.
+	 */
+	int degrees;
+	/* Capture width and height - different from output width and height if 
+	 * rotating 90 or 270 degrees. */
+	int cap_width;
+	int cap_height;
+};
+
 #include "track.h"
 
+#include "netcam_wget.h"
+
+struct url_t {
+	char *service;
+	char *host;
+	int port;
+	char *path;
+};
+
+struct netcam_context {
+	int threadnr;
+	struct rbuf *response;
+	struct url_t url;
+	char *userpass;
+	pthread_mutex_t mutex;
+	int which_image;
+	char *boundary_string;
+	int content_length;
+	int error_frames;
+	struct netcam_image {
+		size_t size;
+		size_t buffer_size;
+		char *buffer;
+		short filled;
+		pthread_cond_t buffer_filled;
+		pthread_mutex_t mutex;
+	} image[2];
+	int (*read)(struct netcam_context *,int);
+};
+
+
 /*
 	these used to be global variables but now each thread will have its
 	own context
@@ -153,12 +227,18 @@ struct context {
 	struct config conf;
 	struct images imgs;
 	struct trackoptions track;
+	struct netcam_context *netcam;		
 	int precap_nr;
 	int precap_cur;
+	int new_img;
+	int preview_shot;
+	
+	struct rotdata rotate_data; /* rotation data is thread-specific */
 
 	int noise;
 	int threshold;
 	int diffs_last[THRESHOLD_TUNE_LENGTH];
+	int smartmask_speed;
 
 	int snapshot;
 	int makemovie;
@@ -175,12 +255,6 @@ struct context {
 	time_t lastshot;
 
 	int lastrate;
-	int filesc;
-	int filesmc;
-	char **files;
-	char **filesm;
-	int *filesrate;
-	int *filesmrate;
 	int moved;
 	int switched;
 	int pause;
@@ -210,15 +284,12 @@ struct context {
 	struct ffmpeg *ffmpeg_new;
 	struct ffmpeg *ffmpeg_motion;
 	struct ffmpeg *ffmpeg_timelapse;
+	struct ffmpeg *ffmpeg_smartmask;
 	char newfilename[255];
 	char motionfilename[255];
 	char timelapsefilename[255];
 #endif
-
-	struct coord predict_lastbox;
 	int last_diffs;
-	struct predict predict;
-	struct predict_set *predict_sets;
 };
 
 extern int threads_running;
