@@ -156,18 +156,46 @@ void put_ppm_bgr24 (FILE *picture, char *image, int width, int height)
 	}
 }
 
+/* copy smartmask as an overlay into motion images and movies */
+void overlay_smartmask (struct context *cnt, unsigned char *out)
+{
+	int i, x, v, width, height, line;
+	struct images *imgs=&cnt->imgs;
+	unsigned char *smartmask=imgs->smartmask_final;
+	
+	i=imgs->motionsize;
+	v=i+((imgs->motionsize)/4);
+	width=imgs->width;
+	height=imgs->height;
+
+	/* set V to 255 to make smartmask appear red */
+	out+=v;
+	for ( i=0; i<height; i+=2){
+		line=i*width;
+		for (x=0; x<width; x+=2){
+			if (smartmask[line+x]==0 ||
+				smartmask[line+x+1]==0 ||
+				smartmask[line+width+x]==0 ||
+				smartmask[line+width+x+1]==0){
+					*out=255;
+			}
+			out++;
+		}
+	}
+}
+
 void put_picture_fd (struct context *cnt, FILE *picture, char *image, int quality)
 {
 	if (cnt->conf.ppm) {
-		put_ppm_bgr24(picture, image, cnt->conf.width, cnt->conf.height);
+		put_ppm_bgr24(picture, image, cnt->imgs.width, cnt->imgs.height);
 	} else {
 //		printf("type: %d %d\n", cnt->imgs.type, VIDEO_PALETTE_YUV420P);
 		switch (cnt->imgs.type) {
 			case VIDEO_PALETTE_YUV420P:
-				put_jpeg_yuv420p(picture, image, cnt->conf.width, cnt->conf.height, quality);
+				put_jpeg_yuv420p(picture, image, cnt->imgs.width, cnt->imgs.height, quality);
 				break;
 			case VIDEO_PALETTE_GREY:
-				put_jpeg_grey(picture, image, cnt->conf.width, cnt->conf.height, quality);
+				put_jpeg_grey(picture, image, cnt->imgs.width, cnt->imgs.height, quality);
 				break;
 		}
 	}
@@ -192,6 +220,6 @@ void put_picture (struct context *cnt, char *file, char *image, int ftype)
 	}
 
 	put_picture_fd(cnt, picture, image, cnt->conf.quality);
-	event(EVENT_FILECREATE, cnt, file, (void *)ftype,NULL);
+	event(EVENT_FILECREATE, cnt, file, (void *)(unsigned long)ftype,NULL);
 	fclose(picture);
 }
