@@ -14,16 +14,25 @@
 struct trackoptions {
 	int dev;
 	/* Config options: */
+	int type;
 	char *port;
 	int motorx;
 	int maxx;
 	int stepsize;
 	int speed;
 	int iomojo_id;
+	int active;
+	int panmin;
+	int panmax;
+	int tiltmin;
+	int tiltmax;
+	int minmaxfound;
 };
 
-int track_center(struct context *);
-int track_move(struct context *, int dev, int pipe, int mpipe, struct coord cent, struct images *imgs);
+extern struct trackoptions track_template;
+
+int track_center(struct context *, int dev, int manual, int xoff, int yoff);
+int track_move(struct context *, int dev, int pipe, int mpipe, struct coord *cent, struct images *imgs, int manual);
 
 /*
 	Some default values:
@@ -31,6 +40,10 @@ int track_move(struct context *, int dev, int pipe, int mpipe, struct coord cent
 #define TRACK_SPEED	255
 #define TRACK_STEPSIZE	40
 
+#define TRACK_TYPE_STEPPER	1
+#define TRACK_TYPE_IOMOJO	2
+#define TRACK_TYPE_PWC		3
+
 /*
 	Some defines for the Serial stepper motor:
  */
@@ -72,4 +85,13 @@ int track_move(struct context *, int dev, int pipe, int mpipe, struct coord cent
 #define IOMOJO_DIRECTION_UP	0x08
 
 
+/*
+	Defines for the Logitech QuickCam Orbit/Sphere USB webcam
+*/
+
+#define LQOS_VERTICAL_DEGREES	180
+#define LQOS_HORIZONAL_DEGREES	120
+
+
+
 #endif /* _INCLUDE_TRACK_H */
