@@ -11,19 +11,114 @@
 #include "motion.h"
 #include "video.h"
 #include "conf.h"
-
-#ifndef WITHOUT_V4L
-#include "pwc-ioctl.h" 
-
-/* for the v4l stuff: */
-#include "sys/mman.h"
-#include "sys/ioctl.h"
-#endif /* WITHOUT_V4L */
-
 /* for rotation */
 #include "rotate.h"
 
 #ifndef WITHOUT_V4L
+
+/* for the v4l stuff: */
+#include "pwc-ioctl.h"
+#include "sys/mman.h"
+#include "sys/ioctl.h"
+#include "math.h"
+#include <sys/utsname.h>
+#include <dirent.h>
+
+static void v4l_picture_controls(struct video_dev *viddev, struct context *cnt)
+{
+	int dev=viddev->fd;
+	unsigned char *image = cnt->imgs.new;
+	struct video_picture vid_pic;
+	int make_change = 0;
+	int i, j = 0, avg = 0, offset = 0;
+
+	if (cnt->conf.contrast && cnt->conf.contrast != viddev->contrast) {
+
+		if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
+			syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+		}
+
+		make_change = 1;
+		vid_pic.contrast = cnt->conf.contrast * 255;
+		viddev->contrast = cnt->conf.contrast;
+	}
+
+	if (cnt->conf.saturation && cnt->conf.saturation != viddev->saturation) {
+
+		if (!make_change) {
+			if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
+				syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+			}
+		}
+
+		make_change = 1;
+		vid_pic.colour = cnt->conf.saturation * 255;
+		viddev->saturation = cnt->conf.saturation;
+	}
+
+	if (cnt->conf.hue && cnt->conf.hue != viddev->hue) {
+
+		if (!make_change) {
+			if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
+				syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+			}
+		}
+
+		make_change = 1;
+		vid_pic.hue = cnt->conf.hue * 255;
+		viddev->hue = cnt->conf.hue;
+	}
+	
+	if (cnt->conf.autobright) {
+		for (i = 0; i < cnt->imgs.width * cnt->imgs.height; i += 100) {
+			avg += *image++;
+			j++;
+		}
+		avg = avg / j;
+
+		if (avg > 140 || avg < 64) {
+			if (!make_change) {
+				if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
+					syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+				}
+			}
+			if (avg > 140) {
+				offset=avg-140;
+				if (vid_pic.brightness > 100*offset) {
+					vid_pic.brightness-=100*offset;
+					make_change = 1;
+				}
+			}
+			if (avg < 64) {
+				offset=64-avg;
+				if (vid_pic.brightness < 65535-100*offset) {
+					vid_pic.brightness+=100*offset;
+					make_change = 1;
+				}
+			}
+		}
+	} else {
+		if (cnt->conf.brightness && cnt->conf.brightness != viddev->brightness) {
+			if (!make_change) {
+				if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
+					syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+				}
+			}
+	
+			make_change = 1;
+			vid_pic.brightness = cnt->conf.brightness * 255;
+			viddev->brightness = cnt->conf.brightness;
+		}
+	}
+
+	if (make_change) {
+		if (ioctl(dev, VIDIOCSPICT, &vid_pic)==-1) {
+			syslog(LOG_ERR, "ioctl (VIDIOCSPICT): %m");
+		}
+	}
+}
+		
+
 void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int height)
 {
 	unsigned char *src, *dest, *src2, *dest2;
@@ -120,6 +215,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	}
 	if (vid_caps.type & VID_TYPE_MONOCHROME) viddev->v4l_fmt=VIDEO_PALETTE_GREY;
 	if (input != IN_DEFAULT) {
+		memset(&vid_chnl, 0, sizeof(struct video_channel));
 		vid_chnl.channel = input;
 		if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
 			syslog(LOG_ERR, "ioctl (VIDIOCGCHAN): %m");
@@ -170,7 +266,6 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 		viddev->v4l_maxbuffer=1;
 		viddev->v4l_curbuffer=0;
 		viddev->v4l_buffers[0]=map;
-		//goto done;
 	}else{
 		map=mmap(0, vid_buf.size, PROT_READ|PROT_WRITE, MAP_SHARED, dev, 0);
 		if (vid_buf.frames>1) {
@@ -225,12 +320,12 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	return map;
 }
 
-static char *v4l_next (struct video_dev *viddev, char *map, int width, int height)
+static char *v4l_next (struct video_dev *viddev, unsigned char *map, int width, int height)
 {
 	int dev=viddev->fd;
 	int frame=viddev->v4l_curbuffer;
 	struct video_mmap vid_mmap;
-	char *cap_map=NULL;
+	unsigned char *cap_map=NULL;
 
 	sigset_t  set, old;
 
@@ -286,7 +381,7 @@ static char *v4l_next (struct video_dev *viddev, char *map, int width, int heigh
 	return map;
 }
 
-void v4l_set_input (struct video_dev *viddev, char *map, int width, int height, int input, int norm, int skip, unsigned long freq)
+void v4l_set_input (struct video_dev *viddev, unsigned char *map, int width, int height, int input, int norm, int skip, unsigned long freq, struct context *cnt)
 {
 	int dev=viddev->fd;
 	int i;
@@ -323,6 +418,7 @@ void v4l_set_input (struct video_dev *viddev, char *map, int width, int height,
 				return;
 			}
 		}
+		v4l_picture_controls(viddev, cnt);
 		viddev->input=input;
 		viddev->width=width;
 		viddev->height=height;
@@ -330,51 +426,132 @@ void v4l_set_input (struct video_dev *viddev, char *map, int width, int height,
 		/* skip a few frames if needed */
 		for (i=0; i<skip; i++)
 			v4l_next (viddev, map, width, height);
+	} else {
+		/* No round robin - we only adjust picture controls */
+		v4l_picture_controls(viddev, cnt);
 	}
 }
 
 static int v4l_open_vidpipe(void)
 {
-	int pipe;
-	FILE *vloopbacks;
+	int pipe = -1;
 	char pipepath[255];
 	char buffer[255];
-	char *loop;
-	char *input;
-	char *istatus;
-	char *output;
-	char *ostatus;
-	
-	vloopbacks=fopen("/proc/video/vloopback/vloopbacks", "r");
-	if (!vloopbacks) {
-		syslog(LOG_ERR, "Failed to open '/proc/video/vloopback/vloopbacks': %m");
+	char *major;
+	char *minor;
+	struct utsname uts;
+
+	if (uname(&uts) < 0) {
+		syslog(LOG_ERR, "Unable to execute uname: %m");
 		return -1;
 	}
-	/* Read vloopback version*/
-	fgets(buffer, 255, vloopbacks);
-	printf("\t%s", buffer);
-	/* Read explaination line */
-	fgets(buffer, 255, vloopbacks);
-	while (fgets(buffer, 255, vloopbacks)) {
-		if (strlen(buffer)>1) {
-			buffer[strlen(buffer)-1]=0;
-			loop=strtok(buffer, "\t");
-			input=strtok(NULL, "\t");
-			istatus=strtok(NULL, "\t");
-			output=strtok(NULL, "\t");
-			ostatus=strtok(NULL, "\t");
-			if (istatus[0]=='-') {
-				sprintf(pipepath, "/dev/%s", input);
-				pipe=open(pipepath, O_RDWR);
-				if (pipe>=0) {
-					printf("\tInput: /dev/%s\n", input);
-					printf("\tOutput: /dev/%s\n", output);
-					return pipe;
+	major = strtok(uts.release, ".");
+	minor = strtok(NULL, ".");
+	if ((major == NULL) || (minor == NULL) || (strcmp(major, "2"))) {
+		syslog(LOG_ERR, "Unable to decipher OS version\n");
+		return -1;
+	}
+	if (strcmp(minor, "5") < 0) {
+		FILE *vloopbacks;
+		char *loop;
+		char *input;
+		char *istatus;
+		char *output;
+		char *ostatus;
+
+		vloopbacks=fopen("/proc/video/vloopback/vloopbacks", "r");
+		if (!vloopbacks) {
+			syslog(LOG_ERR, "Failed to open '/proc/video/vloopback/vloopbacks': %m");
+			return -1;
+		}
+		/* Read vloopback version*/
+		fgets(buffer, 255, vloopbacks);
+		fprintf(stderr,"\t%s", buffer);
+		/* Read explaination line */
+		fgets(buffer, 255, vloopbacks);
+		while (fgets(buffer, 255, vloopbacks)) {
+			if (strlen(buffer)>1) {
+				buffer[strlen(buffer)-1]=0;
+				loop=strtok(buffer, "\t");
+				input=strtok(NULL, "\t");
+				istatus=strtok(NULL, "\t");
+				output=strtok(NULL, "\t");
+				ostatus=strtok(NULL, "\t");
+				if (istatus[0]=='-') {
+					sprintf(pipepath, "/dev/%s", input);
+					pipe=open(pipepath, O_RDWR);
+					if (pipe>=0) {
+						fprintf(stderr,"\tInput: /dev/%s\n", input);
+						fprintf(stderr,"\tOutput: /dev/%s\n", output);
+						break;
+					}
 				}
 			}
 		}
+		fclose(vloopbacks);
+	} else {
+		DIR *dir;
+		struct dirent *dirp;
+		const char prefix[]="/sys/class/video4linux/";
+		char *ptr, *io;
+		int fd;
+		int low=9999;
+		int tfd;
+		int tnum;
+
+		if ((dir=opendir(prefix))== NULL) {
+			syslog(LOG_ERR, "Failed to open '%s': %m", prefix);
+			return -1;
+		}
+		while ((dirp=readdir(dir)) != NULL) {
+			if (!strncmp(dirp->d_name, "video", 5)) {
+				strcpy(buffer, prefix);
+				strcat(buffer, dirp->d_name);
+				strcat(buffer, "/name");
+				if ((fd=open(buffer, O_RDONLY)) >= 0) {
+					if ((read(fd, buffer, sizeof(buffer)-1))<0) {
+						close(fd);
+						continue;
+					}
+					ptr = strtok(buffer, " ");
+					if (strcmp(ptr,"Video")) {
+						close(fd);
+						continue;
+					}
+					major = strtok(NULL, " ");
+					minor = strtok(NULL, " ");
+					io  = strtok(NULL, " \n");
+					if (strcmp(major, "loopback") || strcmp(io, "input")) {
+						close(fd);
+						continue;
+					}
+					if ((ptr=strtok(buffer, " "))==NULL) {
+						close(fd);
+						continue;
+					}
+					tnum = atoi(minor);
+					if (tnum < low) {
+						strcpy(buffer, "/dev/");
+						strcat(buffer, dirp->d_name);
+						if ((tfd=open(buffer, O_RDWR))>=0) {
+							strcpy(pipepath, buffer);
+							if (pipe>=0) {
+								close(pipe);
+							}
+							pipe = tfd;
+							low = tnum;
+						}
+					}
+					close(fd);
+				}
+			}
+		}
+		closedir(dir);
+		if (pipe >= 0) {
+			fprintf(stderr,"Opened input of %s\n", pipepath);
+		}
 	}
-	return -1;
+	return pipe;
 }
 
 static int v4l_startpipe (char *devname, int width, int height, int type)
@@ -418,38 +595,6 @@ static int v4l_putpipe (int dev, char *image, int size)
 	return write(dev, image, size);
 }
 
-static void v4l_autobright (int dev, unsigned char *image, int width, int height)
-{
-	struct video_picture vid_pic;
-	int i, j=0, avg=0, offset=0;
-	
-	for (i=0; i<width*height; i+=100) {
-		avg+=*image++;
-		j++;
-	}
-	avg=avg/j;
-
-	if (avg > 140 || avg < 64) {
-		if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-			syslog(LOG_ERR, "ioctl VIDIOCGPICT: %m");
-		}
-		if (avg > 140) {
-			offset=avg-140;
-			if (vid_pic.brightness > 100*offset)
-				vid_pic.brightness-=100*offset;
-		}
-		if (avg < 64) {
-			offset=64-avg;
-			if (vid_pic.brightness < 65535-100*offset)
-				vid_pic.brightness+=100*offset;
-		}
-		printf("auto_brightness: %d\n", vid_pic.brightness);
-		if (ioctl(dev, VIDIOCSPICT, &vid_pic)==-1) {
-			syslog(LOG_ERR, "ioctl VIDIOCSPICT: %m");
-		}
-	}
-}
-
 /*****************************************************************************
 	Wrappers calling the actual capture routines
  *****************************************************************************/
@@ -478,6 +623,7 @@ void vid_close(void)
 		while(viddevs[++i]) {
 			close(viddevs[i]->fd);
 		}
+		fprintf(stderr,"Closing devices ...\n");
 	}
 }
 
@@ -489,34 +635,43 @@ void vid_cleanup(void)
 			free(viddevs[i]);
 		free(viddevs);
 		viddevs=NULL;
+		fprintf(stderr,"Cleaning viddevs ...\n");
 	}
 }
 #endif /*WITHOUT_V4L*/
 
-int vid_start (struct context *cnt, int width, int height, int input, int norm, unsigned long frequency)
+int vid_start (struct context *cnt)
 {
 	struct config *conf=&cnt->conf;
 	int dev=-1;
 
-#ifdef HAVE_CURL
 	if (conf->netcam_url)
 		return netcam_start(cnt);
-#endif
 
 #ifndef WITHOUT_V4L
 	{
 		int i=-1;
+		int width, height, input, norm;
+		unsigned long frequency;
+
+		/* We use width and height from conf in this function. They will be assigned
+		 * to width and height in imgs here, and cap_width and cap_height in 
+		 * rotate_data won't be set until in rotate_init.
+		 */
+		width = conf->width;
+		height = conf->height;
+		input = conf->input;
+		norm = conf->norm;
+		frequency = conf->frequency;
+		
 		pthread_mutex_lock(&vid_mutex);
 
-		/* If rotation is 90 or 270 degrees then swap width and height. This
-	 	* means that the correct video dimensions will be used, because the
-	 	* width and height were swapped when the configuration was loaded.
-	 	*/
-		if((cnt->conf.rotate_deg == 90) || (cnt->conf.rotate_deg == 270))
-		myswap(width, height);
-
-		cnt->imgs.width=cnt->conf.width;
-		cnt->imgs.height=cnt->conf.height;
+		/* Transfer width and height from conf to imgs. The imgs values are the ones
+		 * that is used internally in Motion. That way, setting width and height via
+		 * xmlrpc won't screw things up.
+		 */
+		cnt->imgs.width=width;
+		cnt->imgs.height=height;
 
 		while (viddevs[++i]) {
 			if (!strcmp(conf->video_device, viddevs[i]->video_device)) {
@@ -541,6 +696,7 @@ int vid_start (struct context *cnt, int width, int height, int input, int norm,
 
 		viddevs=myrealloc(viddevs, sizeof(struct video_dev *)*(i+2), "vid_start");
 		viddevs[i]=mymalloc(sizeof(struct video_dev));
+		memset(viddevs[i], 0, sizeof(struct video_dev));
 		viddevs[i+1]=NULL;
 
 		pthread_mutexattr_init(&viddevs[i]->attr);
@@ -559,12 +715,17 @@ int vid_start (struct context *cnt, int width, int height, int input, int norm,
 		viddevs[i]->height=height;
 		viddevs[i]->width=width;
 		viddevs[i]->freq=frequency;
+		
+		/* We set brightness, contrast, saturation and hue = 0 so that they only get
+		 * set if the config is not zero.
+		 */
+		viddevs[i]->brightness=0;
+		viddevs[i]->contrast=0;
+		viddevs[i]->saturation=0;
+		viddevs[i]->hue=0;
 		viddevs[i]->owner=-1;
 
 		viddevs[i]->v4l_fmt=VIDEO_PALETTE_YUV420P;
-		viddevs[i]->v4l_read_img=0;
-		viddevs[i]->v4l_curbuffer=0;
-		viddevs[i]->v4l_maxbuffer=1;
 
 		if (!v4l_start (viddevs[i], width, height, input, norm, frequency)) {
 			pthread_mutex_unlock(&vid_mutex);
@@ -591,28 +752,26 @@ int vid_start (struct context *cnt, int width, int height, int input, int norm,
 	return dev;
 }
 
-char *vid_next (struct context *cnt, int dev, char *map, int width, int height)
+char *vid_next (struct context *cnt, unsigned char *map)
 {
 	struct config *conf=&cnt->conf;
 	char *ret=NULL;
 
-#ifdef HAVE_CURL
 	if (conf->netcam_url) {
 		int retries=0;
-
-		ret=NULL;
-		while (!ret && retries++<100)
-			ret=netcam_next(cnt, map);
+		while ((!cnt->finish) && (!ret) && (retries++<100)) ret = netcam_next(cnt, map);
 		return ret;
 	}
-#endif
 #ifndef WITHOUT_V4L
 	{
 		int i=-1;
-		/* If rotation is 90 or 270 degrees, then swap width and height. */
-		if((conf->rotate_deg == 90) || (conf->rotate_deg == 270))
-			myswap(width, height);
+		int width, height;
+		int dev = cnt->video_dev;
 
+		/* NOTE: Since this is a capture, we need to use capture dimensions. */
+		width = cnt->rotate_data.cap_width;
+		height = cnt->rotate_data.cap_height;
+		
 		while (viddevs[++i])
 			if (viddevs[i]->fd==dev)
 				break;
@@ -627,7 +786,7 @@ char *vid_next (struct context *cnt, int dev, char *map, int width, int height)
 			cnt->switched=1;
 		}
 
-		v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm, conf->roundrobin_skip, conf->frequency);
+		v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm, conf->roundrobin_skip, conf->frequency, cnt);
 		ret=v4l_next (viddevs[i], map, width, height);
 
 		if (--viddevs[i]->frames <= 0) {
@@ -635,9 +794,9 @@ char *vid_next (struct context *cnt, int dev, char *map, int width, int height)
 			pthread_mutex_unlock(&viddevs[i]->mutex);
 		}
 	
-		/* Rotate the image if specified. */
-		if(conf->rotate_deg > 0) {
-			rotate_map(map, viddevs[i]->v4l_bufsize, width, height, conf->rotate_deg);
+		if(cnt->rotate_data.degrees > 0) {
+			/* rotate the image as specified */
+			rotate_map(map, cnt);
 		}
 	}
 #endif /*WITHOUT_V4L*/
@@ -654,10 +813,4 @@ int vid_putpipe (int dev, char *image, int size)
 {
 	return v4l_putpipe (dev, image, size);
 }
-
-void vid_autobright (struct config *conf, int dev, char *image, int width, int height)
-{
-	if (!conf->netcam_url) 
-		v4l_autobright(dev, image, width, height);
-}
 #endif /*WITHOUT_V4L*/
