@@ -35,6 +35,10 @@ struct video_dev {
 	int input;
 	int width;
 	int height;
+	int brightness;
+	int contrast;
+	int saturation;
+	int hue;
 	unsigned long freq;
 
 	pthread_mutex_t mutex;
@@ -55,21 +59,18 @@ struct video_dev {
 };
 
 /* video functions, video.c */
-int vid_start (struct context *, int width, int height, int input, int norm, unsigned long frequency);
-char *vid_next (struct context *, int dev, char *map, int width, int height);
+int vid_start (struct context *);
+char *vid_next (struct context *, unsigned char *map);
 #ifndef WITHOUT_V4L
 void vid_init(void);
 int vid_startpipe (char *devname, int width, int height, int);
 int vid_putpipe (int dev, char *image, int);
-void vid_autobright (struct config *, int dev, char *image, int width, int height);
 void vid_close(void);
 void vid_cleanup(void);
 #endif
 
-#ifdef HAVE_CURL
 /* Network camera functions, netcam.c */
 int netcam_start (struct context *);
 unsigned char *netcam_next (struct context *, char *);
-#endif /* HAVE_CURL */
 
 #endif /* _INCLUDE_VIDEO_H */
