@@ -11,6 +11,8 @@
 #include "motion.h"
 #include "video_freebsd.h"
 #include "conf.h"
+/* for rotation */
+#include "rotate.h"
 
 #ifndef WITHOUT_V4L
 
@@ -18,9 +20,6 @@
 #include "sys/mman.h"
 #include "sys/ioctl.h"
 
-/* for rotation */
-#include "rotate.h"
-
 /* Hack from xawtv 4.x */
                                                                                                                              
 #define VIDEO_NONE           0
@@ -783,41 +782,6 @@ static int v4l_putpipe (int dev, char *image, int size)
 	return write(dev, image, size);
 }
 
-static void v4l_autobright (int dev, unsigned char *image, int width, int height)
-{
-/* Need to be implemented
-
-	struct video_picture vid_pic;
-	int i, j=0, avg=0, offset=0;
-	
-	for (i=0; i<width*height; i+=100) {
-		avg+=*image++;
-		j++;
-	}
-	avg=avg/j;
-
-	if (avg > 140 || avg < 64) {
-		if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-			syslog(LOG_ERR, "ioctl VIDIOCGPICT: %m");
-		}
-		if (avg > 140) {
-			offset=avg-140;
-			if (vid_pic.brightness > 100*offset)
-				vid_pic.brightness-=100*offset;
-		}
-		if (avg < 64) {
-			offset=64-avg;
-			if (vid_pic.brightness < 65535-100*offset)
-				vid_pic.brightness+=100*offset;
-		}
-		printf("auto_brightness: %d\n", vid_pic.brightness);
-		if (ioctl(dev, VIDIOCSPICT, &vid_pic)==-1) {
-			syslog(LOG_ERR, "ioctl VIDIOCSPICT: %m");
-		}
-	}
-*/
-}
-
 /*****************************************************************************
 	Wrappers calling the actual capture routines
  *****************************************************************************/
@@ -879,31 +843,38 @@ void vid_cleanup(void)
 #endif /*WITHOUT_V4L*/
 
 
-int vid_start (struct context *cnt, int width, int height, int input, int norm, unsigned long frequency)
+int vid_start (struct context *cnt)
 {
 	struct config *conf=&cnt->conf;
         int fd_bktr=-1,fd_tuner=-1;
 	
-#ifdef HAVE_CURL
 	if (conf->netcam_url)
 		return netcam_start(cnt);
-#endif
-
 
 #ifndef WITHOUT_V4L
 	{
 		int i=-1;
+		int width, height, input, norm;
+		unsigned long frequency;
+
+		/* We use width and height from conf in this function. They will be assigned
+		 * to width and height in imgs here, and cap_width and cap_height in 
+		 * rotate_data won't be set until in rotate_init.
+		 */
+		width = conf->width;
+		height = conf->height;
+		input = conf->input;
+		norm = conf->norm;
+		frequency = conf->frequency;
+
 		pthread_mutex_lock(&vid_mutex);
 
- 		/* If rotation is 90 or 270 degrees then swap width and height. This
- 		 * means that the correct video dimensions will be used, because the
- 	 	* width and height were swapped when the configuration was loaded.
- 	 	*/
- 		if((cnt->conf.rotate_deg == 90) || (cnt->conf.rotate_deg == 270))
- 			myswap(width, height);
- 
-		cnt->imgs.width=cnt->conf.width;
-		cnt->imgs.height=cnt->conf.height;
+		/* Transfer width and height from conf to imgs. The imgs values are the ones
+		 * that is used internally in Motion. That way, setting width and height via
+		 * xmlrpc won't screw things up.
+		 */
+		cnt->imgs.width=width;
+		cnt->imgs.height=height;
 
 		while (viddevs[++i]) {
 			if (!strcmp(conf->video_device, viddevs[i]->video_device)) {
@@ -999,11 +970,11 @@ int vid_start (struct context *cnt, int width, int height, int input, int norm,
 	return fd_bktr;
 }
 
-char *vid_next (struct context *cnt, int dev_bktr, char *map, int width, int height)
+char *vid_next (struct context *cnt, char *map)
 {
 	struct config *conf=&cnt->conf;
         char *ret=NULL;
-#ifdef HAVE_CURL
+
 	if (conf->netcam_url) {
 		int retries=0;
 
@@ -1012,14 +983,17 @@ char *vid_next (struct context *cnt, int dev_bktr, char *map, int width, int hei
 			ret=netcam_next(cnt, map);
 		return ret;
 	}
-#endif
+
 #ifndef WITHOUT_V4L
 	{
 		int i=-1;
- 		/* If rotation is 90 or 270 degrees, then swap width and height. */
- 		if((conf->rotate_deg == 90) || (conf->rotate_deg == 270))
- 			myswap(width, height);
-	
+		int width, height;
+		int dev_bktr = cnt->video_dev;
+
+		/* NOTE: Since this is a capture, we need to use capture dimensions. */
+		width = cnt->rotate_data.cap_width;
+		height = cnt->rotate_data.cap_height;
+		
 		while (viddevs[++i])
 			if (viddevs[i]->fd_bktr==dev_bktr)
 				break;
@@ -1044,9 +1018,9 @@ char *vid_next (struct context *cnt, int dev_bktr, char *map, int width, int hei
 			pthread_mutex_unlock(&viddevs[i]->mutex);
 		}
 	
-		// Rotate the image if specified. 
- 		if(conf->rotate_deg > 0) {
- 			rotate_map(map, viddevs[i]->v4l_bufsize, width, height, conf->rotate_deg);
+ 		if(cnt->rotate_data.degrees > 0) {
+			/* rotate the image as specified */
+			rotate_map(map, cnt);
  		}
 	}
 #endif /*WITHOUT_V4L*/
@@ -1064,10 +1038,4 @@ int vid_putpipe (int dev, char *image, int size)
 	return v4l_putpipe (dev, image, size);
 }
 
-void vid_autobright (struct config *conf, int dev, char *image, int height, int width)
-{
-	if (!conf->netcam_url) 
-		v4l_autobright(dev, image, height, width);
-}
-
 #endif /*WITHOUT_V4L*/
