@@ -17,24 +17,25 @@
  *	along with this program; if not, write to the Free Software
  *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-#include <stdio.h>
-#include <unistd.h>
 #include <sys/time.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/ioctl.h>
 #include <ctype.h>
-#include <errno.h>
 #include <sys/fcntl.h>
 #include <sys/stat.h>
-#include <string.h>
 
 #include "picture.h"
 #include "webcam.h"
 #include "motion.h"
 
+/* This function sets up a TCP/IP socket for incomming requests. It is used by two features.
+ * - webcam - called only during initiatization of Motion from the function webcam_init
+ * - xmlrpc - called from the function xmlrpc_httpd_run()
+ * The function sets up a a socket on the port number given by _port_.
+ * If the parameter _local_ is not zero the socket is setup to only accept connects from localhost.
+ * Otherwise any client IP address is accepted. The function returns an integer representing the socket.
+ */
 int http_bindsock(int port, int local)
 {
 	int sl, optval=1;
@@ -224,6 +225,10 @@ int webcam_check_write(struct webcam *list)
 	return write;
 }
 
+/* This function is called from motion.c for each motion thread starting up.
+ * The function setup the incomming tcp socket that the clients connect to
+ * The function returns an integer representing the socket.
+ */
 int webcam_init(struct context *cnt)
 {
 	cnt->webcam.socket=http_bindsock(cnt->conf.webcam_port, cnt->conf.webcam_localhost);
@@ -235,9 +240,9 @@ int webcam_init(struct context *cnt)
 
 void webcam_stop(struct context *cnt)
 {
-	printf("Closing webcam listen socket\n");
+	fprintf(stderr,"Closing webcam listen socket\n");
 	close(cnt->webcam.socket);
-	printf("Closing active webcam sockets\n");
+	fprintf(stderr,"Closing active webcam sockets\n");
 	webcam_close(&cnt->webcam);
 }
 
